package com.indigitall.indigitall_flutter_plugin

import android.Manifest
import android.app.Activity
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.graphics.Color
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.view.View
import android.webkit.WebView
import android.widget.ImageButton
import androidx.annotation.NonNull
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import com.indigitall.android.Configuration
import com.indigitall.android.Indigitall
import com.indigitall.android.callbacks.*
import com.indigitall.android.commons.models.ErrorModel
import com.indigitall.android.commons.utils.CorePreferenceUtils
import com.indigitall.android.inbox.Inbox
import com.indigitall.android.inbox.callbacks.InboxCallback
import com.indigitall.android.models.*
import com.indigitall.android.utils.ServiceUtils
import com.indigitall.indigitall_flutter_plugin.interfaces.IndigitallFirebaseInterface
import com.indigitall.indigitall_flutter_plugin.utils.PushParse
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry
import org.json.JSONArray
import org.json.JSONException
import org.json.JSONObject


/** IndigitallFlutterPlugin */
class IndigitallFlutterPlugin: FlutterPlugin, MethodCallHandler, ActivityAware, IndigitallFirebaseInterface {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel

  private val ACTION_INIT = "initialize"
  private val ACTION_DEVICE_ENABLE = "deviceEnable"
  private val ACTION_DEVICE_DISABLE = "deviceDisable"
  private val ACTION_TOPICS_LIST = "topicsList"
  private val ACTION_TOPICS_SUBSCRIBE = "topicsSubscribe"
  private val ACTION_TOPICS_UNSUBSCRIBE = "topicsUnsubscribe"
  private val ACTION_EXTERNAL_CODE = "setExternalCode"

  private val ACTION_SEND_CUSTOM_EVENT = "sendCustomEvent"
  private val ACTION_GET_PACKAGE_NAME = "packageName"
  private val REQUEST_CODE_ASK_PERMISSIONS = 50001
  private val ACTION_GET_PUSH = "getPush"
  private val ACTION_ON_MESSAGE_RECEIVED = "onMessageReceived"
  private val ACTION_GET_TOKEN = "getToken"
    private val ACTION_IS_INDIGITALL_PUSH_NOTIFICATION = "isIndigitallPushNotification"
  private val ACTION_LOGIN = "logIn"
  private val ACTION_LOGOUT = "logOut"
  private val ACTION_GET_INBOX = "getInbox"
  private val ACTION_GET_MESSAGE_COUNT = "getMessageCount"
  private val ACTION_GET_NEXT_PAGE = "getNextPage"
  private val ACTION_MASSIVE_EDIT = "massiveEditNotificationsWithSendingIdsList"
  private val ACTION_GET_INFO_NOTIFICATION = "getInfoFromNotificationWithSendingId"
  private val ACTION_MODIFY_STATUS = "modifyStatusFromNotificationWithSendingId"
  private val ACTION_GET_AUTH_CONFIG = "getAuthConfig"
  private val ACTION_GET_CUSTOMER = "getCustomer"
  private val ACTION_GET_CUSTOMER_INFORMATION = "getCustomerInformation"
  private val ACTION_UPDATE_VALUE_CUSTOMER_FIELDS = "assignOrUpdateValueToCustomerFields"
  private val ACTION_DELETE_VALUES_CUSTOMER_FIELDS = "deleteValuesFromCustomerFields"
  private val ACTION_LINK = "link"
  private val ACTION_UNLINK = "unlink"

  private val CALLBACK_ON_INDIGITALL_INITIALIZED = "callbackOnIndigitallInitialized"
  private val CALLBACK_ON_NEW_USER_REGISTERED = "callbackOnNewUserRegistered"
  private val CALLBACK_DEVICE_ENABLE = "callbackDeviceEnabled"
  private val CALLBACK_DEVICE_DISABLE = "callbackDeviceDisEnabled"
  private val CALLBACK_TOPIC_LIST = "callbackTopicList"
  private val CALLBACK_TOPIC_SUBSCRIBE = "callbackTopicSubscribe"
  private val CALLBACK_TOPIC_UNSUBSCRIBE = "callbackTopicUnsubscribe"
  private val CALLBACK_EXTERNAL_CODE = "callbackExternalCode"
  private val CALLBACK_SEND_CUSTOM_EVENT = "callbackSendCustomEvent"
  private val CALLBACK_LOGIN = "callbackLogin"
  private val CALLBACK_LOGOUT = "callbackLogout"

  private var context: Context?= null
  private var activityName = ""
  private var activity: Activity?= null

  private var indigitallFlutterInbox: Inbox? = null

  companion object {
    var indigitallInterface: IndigitallFirebaseInterface ?= null
  }

  init {
    indigitallInterface = this
  }

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "indigitall_flutter_plugin")
    channel.setMethodCallHandler(this)
    context = flutterPluginBinding.applicationContext
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
   // binding.addRequestPermissionsResultListener(this)
    activity = binding.activity
    activityName = activity?.javaClass?.name ?: ""
  }

  override fun onDetachedFromActivityForConfigChanges() {
    activity = null
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    //binding.addRequestPermissionsResultListener(this)
    activity = binding.activity
    activityName = activity?.javaClass?.name ?: ""
  }

  override fun onDetachedFromActivity() {
    activity = null
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    when (call.method) {
      "getPlatformVersion" -> {
        result.success("Android ${android.os.Build.VERSION.RELEASE}")
      }
      ACTION_INIT -> {
        initialize(call.arguments, result)
      }
      ACTION_DEVICE_ENABLE -> {
        deviceEnable(result)
      }
      ACTION_DEVICE_DISABLE -> {
        deviceDisable(result)
      }
      ACTION_TOPICS_LIST -> {
        topicsList(result)
      }
      ACTION_TOPICS_SUBSCRIBE -> {
        topicsSubscribe(call.arguments, result)
      }
      ACTION_TOPICS_UNSUBSCRIBE -> {
        topicsUnsubscribe(call.arguments, result)
      }
      ACTION_EXTERNAL_CODE -> {
        setExternalCode(call.arguments, result)
      }
      IndigitallInAppPlugin.ACTION_INAPP_TOPICS_LIST -> {
        IndigitallInAppPlugin(context, activity).inAppTopicsList(result)
      }
      IndigitallInAppPlugin.ACTION_INAPP_TOPICS_SUBSCRIBE -> {
        IndigitallInAppPlugin(context, activity).inAppTopicsSubscribe(call.arguments, result)
      }
      IndigitallInAppPlugin.ACTION_INAPP_TOPICS_UNSUBSCRIBE -> {
        IndigitallInAppPlugin(context, activity).inAppTopicsUnsubscribe(call.arguments, result)
      }
      IndigitallInAppPlugin.ACTION_GET_INAPP -> {
        IndigitallInAppPlugin(context, activity).getInApp(call.arguments, result)
      }
      IndigitallInAppPlugin.ACTION_SHOW_POPUP -> {
        IndigitallInAppPlugin(context, activity).showPopUp(call.arguments, result)
      }
      IndigitallInAppPlugin.ACTION_ON_CLICK -> {
        IndigitallInAppPlugin(context, activity).onClick(call.arguments)
      }
      IndigitallInAppPlugin.ACTION_GET_HTML -> {
        IndigitallInAppPlugin(context, activity).inAppGetHtml(call.arguments, result)
      }
      IndigitallInAppPlugin.ACTION_EVENT_PRINT -> {
        IndigitallInAppPlugin(context, activity).eventPrintInAppRequest(call.arguments)
      }
      IndigitallInAppPlugin.ACTION_ADD_NEW_INAPP_TO_DISMISS_FOREVER -> {
        IndigitallInAppPlugin(context, activity).addNewInAppToDismissForever(call.arguments)
      }
      ACTION_SEND_CUSTOM_EVENT -> {
        sendCustomEvent(call.arguments, result)
      }
      ACTION_GET_PACKAGE_NAME -> {
        getPackageName(result)
      }
      ACTION_LOGIN -> {
        logIn(call.arguments, result)
      }
      ACTION_LOGOUT -> {
        logOut(result)
      }
      ACTION_IS_INDIGITALL_PUSH_NOTIFICATION -> {
        isIndigitallPushNotification(call.arguments, result)
      }
      ACTION_GET_PUSH -> {
        getPush(result)
      }
      ACTION_GET_TOKEN -> {
          getToken(result)
      }
      ACTION_GET_INBOX -> {
        context?.let {
          IndigitallInboxFlutterPlugin(it).getInbox(call.arguments, object : InboxCallback(it) {


            override fun onSuccess(inbox: Inbox) {
              indigitallFlutterInbox = inbox
            }
          }, result)
        }
      }
      ACTION_GET_MESSAGE_COUNT -> {
        IndigitallInboxFlutterPlugin(context).getMessagesCount(call.arguments, result)
      }
      ACTION_GET_NEXT_PAGE -> {
        IndigitallInboxFlutterPlugin(context).getNextPage(indigitallFlutterInbox, call.arguments, result)
      }
      ACTION_MASSIVE_EDIT -> {
        IndigitallInboxFlutterPlugin(context).massiveEdit(indigitallFlutterInbox, call.arguments, result)
      }
      ACTION_GET_INFO_NOTIFICATION -> {
        IndigitallInboxFlutterPlugin(context).getInfoFromNotification(indigitallFlutterInbox, call.arguments, result)
      }
      ACTION_MODIFY_STATUS -> {
        IndigitallInboxFlutterPlugin(context).modifyStatus(indigitallFlutterInbox, call.arguments, result)
      }
//      ACTION_GET_AUTH_CONFIG -> {
//        call.arguments?.let{
//          IndigitallInboxFlutterPlugin(context).jsonAuth = JSONObject(it as Map<*, *>)
//        }
//      }

      ACTION_GET_CUSTOMER -> {
        context?.let{
          IndigitallCustomerFlutterPlugin(it).getCustomer(result)
        }
      }
      ACTION_GET_CUSTOMER_INFORMATION -> {
        context?.let{
          IndigitallCustomerFlutterPlugin(it).getCustomerInformation(call.arguments, result)
        }
      }
      ACTION_LINK -> {
        context?.let{
          IndigitallCustomerFlutterPlugin(it).link(call.arguments, result)
        }
      }
      ACTION_UNLINK -> {
        context?.let{
          IndigitallCustomerFlutterPlugin(it).unlink(call.arguments, result)
        }
      }
      ACTION_UPDATE_VALUE_CUSTOMER_FIELDS -> {
        context?.let{
          IndigitallCustomerFlutterPlugin(it).assignOrUpdateValueToCustomerFields(call.arguments, result)
        }
      }
      ACTION_DELETE_VALUES_CUSTOMER_FIELDS -> {
        context?.let{
          IndigitallCustomerFlutterPlugin(it).deleteValuesFromCustomerFields(call.arguments, result)
        }
      }
      else -> {
        result.notImplemented()
      }
    }
    PluginRegistry.RequestPermissionsResultListener { requestCode, permissions, grantResults ->
      if (requestCode == REQUEST_CODE_ASK_PERMISSIONS && grantResults?.isNotEmpty() == true) {
        permissions?.indices?.let{
          for (i in it) {
            if (Manifest.permission.ACCESS_FINE_LOCATION == permissions[i] && grantResults[i] == PackageManager.PERMISSION_GRANTED) {
              // if (PreferenceUtils.getLocationEnabled(cordova.getActivity().getApplicationContext())) {
              ServiceUtils.registerLocationService(context?.applicationContext)
              // }
              true
            }
          }
        }
      }else{
        false
      }
      false
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    //channel?.setMethodCallHandler(null)
  }

  private fun initialize(arguments: Any?, result: Result) {
    val params = JSONObject(arguments as Map<*, *>)

    val MAP_APP_KEY = "appKey"
    val MAP_SENDER_ID = "senderId"
    val MAP_REQUEST_LOCATION = "requestLocation"
    val MAP_PRODUCT_NAME = "productName"
    val MAP_PRODUCT_VERSION = "productVersion"
    val MAP_WIFI_FILTER = "wifiFilterEnabled"
    val MAP_URL_DEVICE_API = "urlDeviceApi"
    val MAP_URL_INAPP_API = "urlInAppApi"
    val MAP_URL_INBOX_API = "urlInboxApi"

    var appKey: String? = null
    var senderId: String? = null
    var productName: String? = null
    var productVersion: String? = null
    var urlDeviceApi = "https://device-api.indigitall.com/v1"
    var urlInAppApi = "https://inapp-api.indigitall.com/v1"
    var urlInboxApi = "https://inbox-api.indigitall.com/v1"
    var requestLocation = false
    var wifiFilterEnabled = false

    try {
      params?.let{obj ->
        if (obj.has(MAP_APP_KEY)) appKey = obj.getString(MAP_APP_KEY)
        if (obj.has(MAP_SENDER_ID)) senderId = obj.getString(MAP_SENDER_ID)
        if (obj.has(MAP_REQUEST_LOCATION)) requestLocation = obj.getBoolean(MAP_REQUEST_LOCATION)
        if (obj.has(MAP_PRODUCT_NAME)) productName = obj.getString(MAP_PRODUCT_NAME)
        if (obj.has(MAP_PRODUCT_VERSION)) productVersion = obj.getString(MAP_PRODUCT_VERSION)
        if (obj.has(MAP_WIFI_FILTER)) wifiFilterEnabled = obj.getBoolean(MAP_WIFI_FILTER)
        if (obj.has(MAP_URL_DEVICE_API)) urlDeviceApi = obj.getString(MAP_URL_DEVICE_API)
        if (obj.has(MAP_URL_INAPP_API)) urlInAppApi = obj.getString(MAP_URL_INAPP_API)
        if (obj.has(MAP_URL_INBOX_API)) urlInboxApi = obj.getString(MAP_URL_INBOX_API)
      }
    } catch (ex: JSONException) {
      ex.printStackTrace()
    }

    if (requestLocation){
      if (context?.let { ContextCompat.checkSelfPermission(it, Manifest.permission.ACCESS_FINE_LOCATION) } != PackageManager.PERMISSION_GRANTED){
        activity?.let { ActivityCompat.requestPermissions(it, arrayOf(Manifest.permission.ACCESS_FINE_LOCATION), REQUEST_CODE_ASK_PERMISSIONS) }
      }
    }

    if (appKey != null && senderId != null) {
      val config: Configuration = Configuration.Builder(appKey, senderId)
        .setDefaultActivity(activityName)
        .setAutoRequestPermissionLocation(requestLocation)
        .setProductName(productName)
        .setProductVersion(productVersion)
        .wifiFilterEnabled(wifiFilterEnabled)
        .setUrlDeviceApi(urlDeviceApi)
        .setUrlInboxApi(urlInboxApi)
        .build()
      //llamada init
      Indigitall.init(context, config, object : InitCallBack(context) {
        override fun onIndigitallInitialized(permissions: Array<Permission?>?, device: Device?) {
          context?.let {
            ContextCompat.getMainExecutor(it).execute {
              result.success(device?.let { PushParse.mapFromDevice(it, CALLBACK_ON_INDIGITALL_INITIALIZED)})
            }
          }
        }

        override fun onNewUserRegistered(device: Device?) {
          context?.let {
            ContextCompat.getMainExecutor(it).execute {
              result.success(device?.let { PushParse.mapFromDevice(it, CALLBACK_ON_NEW_USER_REGISTERED) })
            }
          }
        }

        override fun onErrorInitialized(p0: ErrorModel?) {
          context?.let {
            ContextCompat.getMainExecutor(it).execute {
              result.error("onErrorInitialized", p0?.errorMessage?.errorMessage, null)
            }
          }
        }
      })
    }
  }

  private fun getPackageName(result: Result) {
    context?.let {
      ContextCompat.getMainExecutor(context!!).execute {
        result.success(PushParse.stringToMap("packageName", it.packageName))
      }
    }
  }

  private fun deviceEnable(result: Result) {
    Indigitall.deviceEnable(context, object : DeviceCallback(context) {
      override fun onSuccess(device: Device?) {
        context?.let {
          ContextCompat.getMainExecutor(it).execute {
            result.success(device?.let { PushParse.mapFromDevice(it, CALLBACK_DEVICE_ENABLE) })
          }
        }
      }

      override fun onFail(error: ErrorModel?) {
        context?.let {
          ContextCompat.getMainExecutor(it).execute {
            error?.errorCode?.name?.let{result.error(it, error.errorMessage.errorMessage, null)}
          }
        }
      }
    })
  }

  private fun deviceDisable(result: Result) {
    Indigitall.deviceDisable(context, object : DeviceCallback(context) {
      override fun onSuccess(device: Device?) {
        context?.let {
          ContextCompat.getMainExecutor(it).execute {
            result.success(device?.let { PushParse.mapFromDevice(it, CALLBACK_DEVICE_DISABLE) })
          }
        }
      }

      override fun onFail(error: ErrorModel?) {
        context?.let {
          ContextCompat.getMainExecutor(it).execute {
            error?.errorCode?.name?.let{result.error(it, error.errorMessage.errorMessage, null)}
          }
        }
      }
    })
  }

  private fun topicsList(result: Result) {
    Indigitall.topicsList(context, object : TopicsCallback() {
      override fun onSuccess(topics: Array<Topic>) {
        context?.let {
          ContextCompat.getMainExecutor(it).execute {
            result.success(PushParse.topicToMap(CALLBACK_TOPIC_LIST, PushParse.listFromTopicArray(topics)))
          }
        }
      }

      override fun onFail(error: ErrorModel?) {
        context?.let {
          ContextCompat.getMainExecutor(it).execute {
            error?.errorCode?.name?.let{result.error(it, error.errorMessage.errorMessage, null)}
          }
        }
      }
    })
  }

  private fun topicsSubscribe(
    arguments: Any?,
    result: Result
  ) {
    arguments?.let{
      val jsonArray = JSONArray(it as List<*>)
      val topics = PushParse.parseTopicArray(jsonArray)
      Indigitall.topicsSubscribe(context, topics, object : TopicsCallback() {
        override fun onSuccess(topics: Array<Topic>) {
          context?.let { it1 ->
            ContextCompat.getMainExecutor(it1).execute {
              result.success(PushParse.topicToMap(CALLBACK_TOPIC_SUBSCRIBE, PushParse.listFromTopicArray(topics)))
            }
          }
        }

        override fun onFail(error: ErrorModel?) {
          context?.let { it1 ->
            ContextCompat.getMainExecutor(it1).execute {
              error?.errorCode?.name?.let{result.error(it, error.errorMessage.errorMessage, null)}
            }
          }
        }
      })
    }
  }

  private fun topicsUnsubscribe(
    arguments: Any?,
    result: Result
  ) {
    arguments?.let{
      val jsonArray = JSONArray(it as List<*>)
      val topics = PushParse.parseTopicArray(jsonArray)
      Indigitall.topicsUnsubscribe(context, topics, object : TopicsCallback() {
        override fun onSuccess(topics: Array<Topic>) {
          context?.let { it1 ->
            ContextCompat.getMainExecutor(it1).execute {
              result.success(PushParse.topicToMap(CALLBACK_TOPIC_UNSUBSCRIBE, PushParse.listFromTopicArray(topics)))
            }
          }
        }

        override fun onFail(error: ErrorModel?) {
          context?.let { it1 ->
            ContextCompat.getMainExecutor(it1).execute {
              error?.errorCode?.name?.let{result.error(it, error.errorMessage.errorMessage, null)}
            }
          }
        }
      })
    }
  }

  @Throws(JSONException::class)
  private fun setExternalCode(
    arguments: Any?,
    result: Result
  ) {
    arguments?.let {
      val params = arguments as String
      Indigitall.setExternalCode(
        context,
        params,
        object : DeviceCallback(context) {
          override fun onSuccess(device: Device?) {
            context?.let { it1 ->
              ContextCompat.getMainExecutor(it1).execute {
                result.success(device?.let { PushParse.mapFromDevice(it, CALLBACK_EXTERNAL_CODE) })
              }
            }
          }

          override fun onFail(error: ErrorModel?) {
            context?.let { it1 ->
              ContextCompat.getMainExecutor(it1).execute {
                error?.errorCode?.name?.let{result.error(it, error.errorMessage.errorMessage, null)}
              }
            }
          }
        })
    }
  }

  private fun isIndigitallPushNotification(arguments: Any?, result: Result) {
      var isIndigitallPush = false
    arguments?.let{
      val thread = Thread {
        try {
          val json = JSONObject(arguments as Map<*, *>)
          try {

            val push = Push(context, json)

              isIndigitallPush = push?.appKey != null && push.appKey == CorePreferenceUtils.getAppKey(context)
          } catch (ex: JSONException) {
            ex.printStackTrace()
              isIndigitallPush = false
          }
        } catch (e: Exception) {
          e.printStackTrace()
        }
        context?.let { it1 ->
          ContextCompat.getMainExecutor(it1).execute {
            result.success(isIndigitallPush)
          }
        }
      }
      thread.start()
    }
  }

  @Throws(JSONException::class)
  private fun getPush(result: Result) {
    activity?.intent?.let{
      it.extras?.let {itExtras ->
        val extras: Bundle = itExtras
        if (extras.containsKey(Push.EXTRA_PUSH)) {
          var push: Push? = null
          push = if (context != null) {
            Push(context!!, extras.getString(Push.EXTRA_PUSH))
          } else {
            Push(extras.getString(Push.EXTRA_PUSH))
          }
          context?.let { it1 ->
            ContextCompat.getMainExecutor(it1).execute {
              result.success(PushParse.mapFromPush(push))
            }
          }
        }
      }
    }
  }

    private fun getToken(result: Result) {
        ServiceUtils.getPushToken(context, object: PushTokenCallback(){

          override fun onError(error: PushErrorModel) {
            context?.let {
              ContextCompat.getMainExecutor(it).execute {
                error.errorCode.name.let{result.error(it, error.errorMessage.errorMessage, null)}
              }
            }
          }

          override fun onSuccess(token: String?) {
            context?.let {
              ContextCompat.getMainExecutor(it).execute {
                val map: MutableMap<String, Any> = HashMap()
                token?.let{ map["token"] =token }
                result.success(map)
              }
            }
            }

        })
    }

    override fun onIndigitallMessageReceived(context: Context?, json: JSONObject?) {
      if (context != null) {
        ContextCompat.getMainExecutor(context).execute {
          json?.let {
  //            if (notificationResult != null) {
            val map: MutableMap<String, Any> = HashMap()
            map["notification"] = it.toString()
            channel.invokeMethod(ACTION_ON_MESSAGE_RECEIVED, map)
            //notificationResult?.success(map)
  //            } else {
  //                context?.let { it1 -> FirebaseUtils.showNotification(it1, it) }
  //            }
          }
        }
      }
    }

  @Throws(JSONException::class)
  private fun sendCustomEvent(
    arguments: Any?,
    result: Result
  ) {
    arguments?.let{
      val obj = JSONObject(arguments as Map<*, *>)
      val MAP_EVENT = "event"
      val MAP_EVENT_DATA = "customData"
      var eventCustom: String? = null
      var eventCustomData: JSONObject ?= null

      if (obj.has(MAP_EVENT)) eventCustom = obj.getString(MAP_EVENT)
      if (obj.has(MAP_EVENT_DATA)) eventCustomData = JSONObject(obj.getString(MAP_EVENT_DATA))

      context?.let { it ->
        Indigitall.sendCustomEvent(
          it,
          eventCustom,
          eventCustomData,
          object : EventCallback(it) {
            override fun onSuccess() {
              ContextCompat.getMainExecutor(context).execute {
                result.success(PushParse.stringToMap(CALLBACK_SEND_CUSTOM_EVENT, "event success"))
              }
            }

            override fun onError(error: ErrorModel) {
              ContextCompat.getMainExecutor(context).execute {
                result.error(error.errorCode.name, error.errorMessage.errorMessage, null)
              }
            }
          })
      }
    }

  }


  @Throws(JSONException::class)
  private fun logIn(arguments: Any?, result: Result) {
    arguments?.let{
      val params = it as String
      Indigitall.logIn(context, params, object : DeviceCallback(context) {
        override fun onSuccess(device: Device?) {
          context?.let { it1 ->
            ContextCompat.getMainExecutor(it1).execute {
              result.success(device?.let { PushParse.mapFromDevice(it, CALLBACK_LOGIN) })
            }
          }
        }

        override fun onFail(error: ErrorModel) {
          context?.let { it1 ->
            ContextCompat.getMainExecutor(it1).execute {
              result.error(error.errorCode.name, error.errorMessage.errorMessage, null)
            }
          }
        }
      })
    }
  }

  @Throws(JSONException::class)
  private fun logOut(result: Result) {
    Indigitall.logOut(context, object : DeviceCallback(context) {
      override fun onSuccess(device: Device?) {
        context?.let {
          ContextCompat.getMainExecutor(it).execute {
            result.success(device?.let { PushParse.mapFromDevice(it, CALLBACK_LOGOUT) })
          }
        }
      }

      override fun onFail(error: ErrorModel?) {
        context?.let {
          ContextCompat.getMainExecutor(it).execute {
            error?.errorCode?.name?.let{result.error(it, error.errorMessage.errorMessage, null)}
          }
        }
      }
    })
  }



}
