package com.indigitall.indigitall_flutter_plugin

import android.annotation.SuppressLint
import android.app.Activity
import android.content.Context
import android.graphics.Color
import android.os.AsyncTask
import android.view.View
import android.webkit.WebView
import android.widget.ImageButton
import androidx.core.content.ContextCompat
import com.indigitall.android.commons.callbacks.BaseCallback
import com.indigitall.android.commons.models.CoreAction
import com.indigitall.android.commons.models.ErrorModel
import com.indigitall.android.commons.models.LogLevel
import com.indigitall.android.commons.utils.CorePreferenceUtils
import com.indigitall.android.commons.utils.DeviceUtils
import com.indigitall.android.commons.utils.Log
import com.indigitall.android.inapp.InAppConfiguration
import com.indigitall.android.inapp.InAppConstants
import com.indigitall.android.inapp.InAppIndigitall
import com.indigitall.android.inapp.Utils.*
import com.indigitall.android.inapp.callbacks.GetInAppCallback
import com.indigitall.android.inapp.callbacks.InAppCallback
import com.indigitall.android.inapp.callbacks.InAppTopicCallback
import com.indigitall.android.inapp.callbacks.ShowInAppCallback
import com.indigitall.android.inapp.models.InApp
import com.indigitall.android.inapp.models.InAppAction
import com.indigitall.android.inapp.models.InAppErrorModel
import com.indigitall.android.utils.PushPreferenceUtils
import com.indigitall.indigitall_flutter_plugin.utils.InAppParse
import io.flutter.plugin.common.MethodChannel.Result
import org.json.JSONArray
import org.json.JSONException
import org.json.JSONObject
import java.io.BufferedReader
import java.io.IOException
import java.io.InputStreamReader
import java.net.URL

class IndigitallInAppPlugin(val context: Context?, private val activity: Activity?) {
    companion object {
        val ACTION_SHOW_POPUP = "showPopUp"
        val ACTION_GET_INAPP = "getInApp"
        val ACTION_ON_CLICK = "onClick"
        val ACTION_ADD_NEW_INAPP_TO_DISMISS_FOREVER = "addNewInAppToDismissForever"
        val ACTION_GET_HTML = "inAppGetHtml"
        val ACTION_EVENT_PRINT = "eventPrintInAppRequest"
        val ACTION_INAPP_TOPICS_LIST = "inAppTopicsList"
        val ACTION_INAPP_TOPICS_SUBSCRIBE = "inAppTopicsSubscribe"
        val ACTION_INAPP_TOPICS_UNSUBSCRIBE = "inAppTopicsUnsubscribe"
    }
    private val INDIGITAL_INAPP_PLUGIN = "IndigitallFlutterPlugin"

    private val CALLBACK_TOPIC_LIST = "callbackTopicList"
    private val CALLBACK_TOPIC_SUBSCRIBE = "callbackTopicSubscribe"
    private val CALLBACK_TOPIC_UNSUBSCRIBE = "callbackTopicUnsubscribe"
    private val CALLBACK_DID_DISMISSED = "callbackDidDismissed"
    private val CALLBACK_DID_CLOSED = "callbackDidClosed"
    private val CALLBACK_DID_CLICKED = "callbackDidClicked"
    private val CALLBACK_ON_ERROR = "callbackOnError"

    private val MAP_APP_KEY = "appKey"
    private val MAP_URL_INAPP_API = "domainInApp"
    private val MAP_INAPP_CODE = "inAppCode"
    private val MAP_POPUP_CODE = "popUpCode"
    private val CLOSE_BUTTON = "closeButton"
    private val CLOSE_ICON_DISABLED = "closeIconDisabled"

    fun getInApp(arguments: Any?, result: Result){
        arguments?.let{ arg ->
            val params = JSONObject(arg as Map<*, *>)

            var appKey: String?= null
            var domainInApp = "https://device-api.indigitall.com/v2"
            var inAppCode: String?= null
            CorePreferenceUtils.getApiURL(context)?.let{
                domainInApp = it
            }

            try {
                if (params.has(MAP_APP_KEY)) appKey = params.getString(MAP_APP_KEY)
                if (params.has(MAP_URL_INAPP_API)) domainInApp = params.getString(MAP_URL_INAPP_API)
                if (params.has(MAP_INAPP_CODE)) inAppCode = params.getString(MAP_INAPP_CODE)
            } catch (ex: JSONException) {
                ex.printStackTrace()
            }

            if (inAppCode != null && appKey != null) {
                context?.let {
                    val inAppConfiguration = InAppConfiguration.Builder(appKey)
                    inAppConfiguration.setUrlInAppApi(domainInApp)
                    inAppConfiguration.setLogDebug(LogLevel.DEBUG)
                    val config: InAppConfiguration = inAppConfiguration.build()
                    ServiceUtils.registerServices(it, config)
                    config.updateConfiguration(it)
                    ShowInAppUtils.inAppWasGot(it, inAppCode, object : GetInAppCallback() {
                        override fun didClickOut(inApp: InApp, errorModel: InAppErrorModel) {
                            InAppParse.mapInAppError(it, errorModel, result)
                        }

                        override fun didDismissForever(inApp: InApp, errorModel: InAppErrorModel) {
                            InAppParse.mapInAppError(it, errorModel, result)
                        }

                        override fun didExpired(inApp: InApp, errorModel: InAppErrorModel) {
                            InAppParse.mapInAppError(it, errorModel, result)
                        }

                        override fun didFound(inApp: InApp) {
                            ContextCompat.getMainExecutor(it).execute {
                                result.success(InAppParse.inAppToMap(inApp))
                            }
                        }

                        override fun didShowManyTimes(inApp: InApp, errorModel: InAppErrorModel) {
                            InAppParse.mapInAppError(it, errorModel, result)
                        }

                        override fun notFound() {
                            InAppIndigitall.inAppGet(it, inAppCode, object : InAppCallback(it) {
                                override fun onSuccess(inApp: InApp?) {
                                    ContextCompat.getMainExecutor(it).execute {
                                        result.success(inApp?.let { it1 -> InAppParse.inAppToMap(it1) })
                                    }
                                }

                                override fun onFail(errorModel: ErrorModel?) {
                                    super.onFail(errorModel)
                                    InAppParse.mapError(it, errorModel, result)
                                }
                            })
                        }
                    })
                }
            }
        }
    }

    fun inAppGetHtml(arguments: Any?, result: Result) {
        arguments?.let { it1 ->
            val url = it1 as String
            val asyncTask: AsyncTask<Void?, Void?, String?> = @SuppressLint("StaticFieldLeak")
            object : AsyncTask<Void?, Void?, String?>() {
                override fun onPostExecute(html: String?) {
                    super.onPostExecute(html)
                    //TODO: with form "IndigitallInApp.onClick" has to change to InAppIndigitallInterface.onClick
                    val htmlReplace = html?.replace("IndigitallInApp.onClick","${INDIGITAL_INAPP_PLUGIN}.onClick")
                    val htmlFinal = "${htmlReplace}${InAppConstants.SCRIPT}"
                    context?.let {
                        ContextCompat.getMainExecutor(it).execute {
                            result.success(htmlFinal)
                        }
                    }
                }

                override fun doInBackground(vararg params: Void?): String? {
                    val html: String = url
                    var finalHtml = ""
                    var htmlURL: URL? = null
                    try {
                        htmlURL = URL(html)
                        val `in` = BufferedReader(
                            InputStreamReader(
                                htmlURL.openStream()
                            )
                        )
                        var inputLine: String
                        while (`in`.readLine().also { inputLine = it } != null) finalHtml += inputLine
                        `in`.close()
                    } catch (e: IOException) {
                        e.printStackTrace()
                    }
                    //                    log.d("InAppPopUp html: " + html).writeLog();
                    return finalHtml
                }
            }
            asyncTask.execute()
        }

    }

    fun onClick(arguments: Any?) {
        try {
            val params = JSONObject(arguments as Map<*, *>)
            var inApp: InApp? = null
            var inAppAction: InAppAction? = null
            if (params.has("inApp")) {
                inApp = InApp(params.getJSONObject("inApp"))
            }
            if (params.has("action")) inAppAction = InAppAction(params.getJSONObject("action"))
            inApp?.let{
                it.properties?.action = inAppAction
                val intent = it.properties.action.getIntent(context)
                val isAppAction = it.properties.action.type === CoreAction.Type.APP
                context?.let { it1 -> InAppUtils.inAppWasClicked(it1, it, it.properties.action, intent, isAppAction) }
                eventClickInAppRequest(it)
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    fun addNewInAppToDismissForever(arguments: Any?) {
        try {
            val obj = JSONObject(arguments as Map<*, *>)
            var inApp = InApp(obj)
            inApp?.let{
                context?.let { it1 -> PopUpUtils.addNewInAppToDismissForever(it1, it) }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    fun eventPrintInAppRequest(arguments: Any?) {
        val obj = JSONObject(arguments as Map<*, *>)
        var inApp = InApp(obj)
        context?.let { EventUtils.sendEventPrint(it, inApp) }
    }

    fun inAppTopicsList(result: Result) {
        InAppIndigitall.topicsList(context, object: BaseCallback(){
            override fun onSuccess(p0: JSONObject?) {
                context?.let {
                    ContextCompat.getMainExecutor(it).execute {
                        result.success(InAppParse.topicToMap(CALLBACK_TOPIC_LIST, p0.toString()))
                    }
                }
            }

            override fun onFail(errorModel: ErrorModel?) {
                context?.let { InAppParse.mapError(it, errorModel, result) }
            }
        })
    }

    fun inAppTopicsSubscribe(arguments: Any?, result: Result) {
        arguments?.let { arg ->
            val topics = InAppParse.parseTopicArray(JSONArray(arg as List<*>))
            InAppIndigitall.topicsSubscribe(context, topics, object : InAppTopicCallback() {
                override fun onError(error: InAppErrorModel) {
                    context?.let { InAppParse.mapInAppError(it, error, result) }
                }

                override fun onSuccess(topicCodes: String) {
                    context?.let {
                        ContextCompat.getMainExecutor(it).execute {
                            result.success(InAppParse.topicToMap(CALLBACK_TOPIC_SUBSCRIBE, topicCodes))
                        }
                    }
                }
            })
        }
    }

    fun inAppTopicsUnsubscribe(arguments: Any?, result: Result) {
        arguments?.let { arg ->
            val topics = InAppParse.parseTopicArray(JSONArray(arg as List<*>))
            InAppIndigitall.topicsUnsubscribe(context, topics, object : InAppTopicCallback() {
                override fun onError(error: InAppErrorModel) {
                    context?.let { InAppParse.mapInAppError(it, error, result) }
                }

                override fun onSuccess(topicCodes: String) {
                    context?.let {
                        ContextCompat.getMainExecutor(it).execute {
                            result.success(InAppParse.topicToMap(CALLBACK_TOPIC_UNSUBSCRIBE, topicCodes))
                        }
                    }
                }
            })
        }
    }

    fun showPopUp(arguments: Any?, result: Result) {
        arguments?.let {
            context?.let{ contxt ->
                val obj = JSONObject(it as Map<*, *>)
//      val view = obj.get(VIEW) as View
                val view = activity?.findViewById<View>(android.R.id.content)?.rootView
                var inAppCode: String?= null
                var imageCloseButtonName: String?= null
                var imageCloseButton: ImageButton?= null
                var closeIconDisabled = false
                var appKey = CorePreferenceUtils.getAppKey(contxt)
                var domainInApp = "https://device-api.indigitall.com/v2"

                if (obj.has(CLOSE_BUTTON)){
                    imageCloseButtonName = obj.get(CLOSE_BUTTON) as String
                    imageCloseButton = ImageButton(contxt)
                    imageCloseButton.setBackgroundColor(Color.TRANSPARENT)
                    imageCloseButton.setImageResource(contxt.resources.getIdentifier(imageCloseButtonName, "drawable", contxt.packageName))
                }
                if (obj.has(MAP_APP_KEY)) appKey = obj.getString(MAP_APP_KEY)
                if (obj.has(MAP_URL_INAPP_API)) domainInApp = obj.getString(MAP_URL_INAPP_API).toString()
                if (obj.has(CLOSE_ICON_DISABLED)) closeIconDisabled = obj.getBoolean(CLOSE_ICON_DISABLED)
                inAppCode = obj.getString(MAP_POPUP_CODE)
                inAppCode?.let { code ->
                    setExternalCodeAndDeviceId(appKey)
                    val inAppConfiguration = InAppConfiguration.Builder(appKey)
                    inAppConfiguration.setUrlInAppApi(domainInApp)
                    val config: InAppConfiguration = inAppConfiguration.build()

                    ServiceUtils.registerServices(contxt, config)
                    config.updateConfiguration(contxt)

                    InAppIndigitall.showPopUp(
                        config,
                        context,
                        view,
                        inAppCode,
                        imageCloseButton,
                        closeIconDisabled,
                        object: ShowInAppCallback(){
                            override fun onLoad(p0: String?, p1: WebView?) {

                                ContextCompat.getMainExecutor(contxt).execute {
//                    result.success(InAppParse.inAppToMap(POP_UP_CODE, p0))
                                }
                            }

                            override fun onFail(p0: String?, p1: WebView?, p2: String?) {
                                ContextCompat.getMainExecutor(contxt).execute {
                                    result.error(CALLBACK_ON_ERROR, p0, null)
                                }

                            }

                            override fun onShowTimeFinished(p0: String?, p1: WebView?, showTime: Int) {
                                val log = Log("IndigitallFlutterPlugin", context)
                                log.d("onShowTimeFinished").writeLog()
                            }

                            override fun didDismissed() {
                                super.didDismissed()

                                ContextCompat.getMainExecutor(contxt).execute {
                                    result.error(CALLBACK_DID_DISMISSED, "popUP didDismissed", null)
                                }

                            }

                            override fun didClicked() {
                                super.didClicked()

                                ContextCompat.getMainExecutor(contxt).execute {
                                    result.error(CALLBACK_DID_CLICKED, "popUp didClicked", null)
                                }
                            }

                            override fun didClosed() {
                                super.didClosed()
                                ContextCompat.getMainExecutor(contxt).execute {
                                    result.error(CALLBACK_DID_CLOSED, "popUp didClosed", null)
                                }

                            }

                            override fun didExpired(inApp: InApp?, errorModel: InAppErrorModel?) {
                                super.didExpired(inApp, errorModel)
                                InAppParse.mapInAppError(context, errorModel, result)
                            }

                            override fun didClickOut(inApp: InApp?, errorModel: InAppErrorModel?) {
                                super.didClickOut(inApp, errorModel)
                                InAppParse.mapInAppError(context, errorModel, result)
                            }

                            override fun didShowManyTimes(inApp: InApp?, errorModel: InAppErrorModel?) {
                                super.didShowManyTimes(inApp, errorModel)
                                InAppParse.mapInAppError(context, errorModel, result)
                            }

                            override fun didDismissForever(inApp: InApp?, errorModel: InAppErrorModel?) {
                                super.didDismissForever(inApp, errorModel)
                                InAppParse.mapInAppError(context, errorModel, result)
                            }

                            override fun onSuccess(inApp: InApp?) {
                                super.onSuccess(inApp)
                                ContextCompat.getMainExecutor(contxt).execute {
                                    result.success(inApp?.let { it1 -> InAppParse.inAppToMap(it1) })
                                }

                            }
                        }
                    )
                }
            }
        }
    }

    fun eventClickInAppRequest(inApp: InApp?) {
        context?.let { EventUtils.sendEventClick(it, inApp) }
    }

    fun setExternalCodeAndDeviceId(appKey: String) {
        var deviceId = CorePreferenceUtils.getDeviceId(context)
        if (deviceId.isNullOrEmpty()) {
            deviceId = EventUtils.generateId()
            CorePreferenceUtils.setDeviceId(context, deviceId)
        }
        if (CorePreferenceUtils.getExternalId(context).isNullOrEmpty()) {
            CorePreferenceUtils.setExternalId(context, DeviceUtils.hMac(
                "HmacSHA256",
                appKey,
                deviceId
            ))
        }
    }

}