package com.indigitall.indigitall_flutter_plugin

import android.content.Context
import androidx.core.content.ContextCompat
import com.indigitall.android.commons.models.*

import io.flutter.plugin.common.MethodChannel.Result
import org.json.JSONException
import org.json.JSONObject


import org.json.JSONArray

import com.indigitall.android.inbox.Inbox
import com.indigitall.android.inbox.Utils.InboxAuthListener
import com.indigitall.android.inbox.callbacks.InboxNotificationsCallback
import com.indigitall.android.inbox.callbacks.InboxBaseCallback
import com.indigitall.android.inbox.callbacks.InboxNextPageCallback
import com.indigitall.android.inbox.callbacks.InboxCountersCallback
import com.indigitall.android.inbox.callbacks.InboxCallback
import com.indigitall.android.inbox.models.*
import com.indigitall.android.models.Push
import com.indigitall.android.models.PushAction
import com.indigitall.android.models.PushButton
import com.indigitall.indigitall_flutter_plugin.utils.InboxParse
import com.indigitall.indigitall_flutter_plugin.utils.PushParse


/** IndigitallInboxFlutterPlugin */
class IndigitallInboxFlutterPlugin(val context: Context?) : InboxAuthListener {

  private val AUTH_VAR = "auth"
  private val CALLBACK_GET_INBOX = "callbackGetInbox"
  private val CALLBACK_GET_NEXT_PAGE = "callbackGetNextPage"
  private val CALLBACK_MASSIVE_EDIT = "callbackMassiveEdit"
  private val CALLBACK_GET_INFO_NOTIFICATION = "callbackGetInfoNotification"
  private val CALLBACK_MODIFY_STATUS = "callbackModifyStatus"


  var jsonAuth: JSONObject ?= null


  override fun getAuthConfig(): JSONObject {
    var json = JSONObject()
    jsonAuth?.let{
      json = it
    }
    return json
  }

  fun getInbox(arguments: Any?, inboxCallback: InboxCallback ,result: Result){
    context?.let {
      arguments?.let { args ->
        jsonAuth = JSONObject(args as Map<*, *>)
      }
      Inbox.Companion.getInbox(it, object : InboxCallback(it) {
        override fun onSuccess(inbox: Inbox) {
          ContextCompat.getMainExecutor(context).execute {
            inboxCallback.onSuccess(inbox)
            val inboxMap: MutableMap<String, Map<*,*>> = HashMap()
            inboxMap[CALLBACK_GET_INBOX] = InboxParse.mapFromInbox(inbox, null)
            result.success(inboxMap)
          }
        }

        override fun onFail(error: ErrorModel) {
          ContextCompat.getMainExecutor(context).execute {
            result.error(error.errorCode.name, error.errorMessage.errorMessage, null)
          }
        }

      })
    }
  }

  fun getMessagesCount(arguments: Any?, result: Result){
    context?.let {
      arguments?.let { args ->
        jsonAuth = JSONObject(args as Map<*, *>)
      }
      Inbox.Companion.getMessagesCount(it, object : InboxCountersCallback(it) {
        override fun onSuccess(inboxCounters: InboxCounters?) {
          ContextCompat.getMainExecutor(context).execute {
            result.success(InboxParse.mapFromMessagesCount(inboxCounters))
          }

        }

        override fun onFail(error: ErrorModel) {
          ContextCompat.getMainExecutor(context).execute {
            result.error(error.errorCode.name, error.errorMessage.errorMessage, null)
          }
        }
      })
    }
  }

  fun getNextPage(myInbox: Inbox?, arguments: Any?, result: Result){
    context?.let {
      arguments?.let { args ->
        jsonAuth = JSONObject(args as Map<*, *>)
      }
      myInbox?.getNextPage(it, object : InboxNextPageCallback(it) {
        override fun onError(
          errorId: Int?,
          errorMessage: String?,
          descriptionMessage: String?
        ) {
          ContextCompat.getMainExecutor(context).execute {
            result.error(errorId.toString(), errorMessage, descriptionMessage)
          }
        }
        override fun onSuccess(inbox: Inbox, arrayList: ArrayList<InboxNotification>?) {
          ContextCompat.getMainExecutor(context).execute {
            val inboxMap: MutableMap<String, Map<*,*>> = HashMap()
            inboxMap[CALLBACK_GET_NEXT_PAGE] = InboxParse.mapFromInbox(inbox, arrayList)
            result.success(inboxMap)
          }
        }
      })
    }
  }

  fun massiveEdit(myInbox: Inbox?, arguments: Any?, result: Result){
    val MAP_SENDING_ID_LIST = "sendingIdList"
    val MAP_STATUS = "status"
    var status: String? = null
    var jsonArraySendingIdList: JSONArray? = null
    val sendingIdList = ArrayList<Int>()
    arguments?.let{args ->
      val params = JSONObject(args as Map<*, *>)
      try {
        if (params.has(AUTH_VAR)) jsonAuth = params.getJSONObject(AUTH_VAR)
        if (params.has(MAP_SENDING_ID_LIST)) jsonArraySendingIdList =
            params.getJSONArray(MAP_SENDING_ID_LIST)
        if (params.has(MAP_STATUS)) status = params.getString(MAP_STATUS)

        if (jsonArraySendingIdList != null) {
          for (i in 0 until jsonArraySendingIdList!!.length()) {
            sendingIdList.add(jsonArraySendingIdList!!.getInt(i))
          }
        }
      } catch (ex: JSONException) {
        ex.printStackTrace()
      }
      var inboxStatus: InboxStatus? = null
      if (status != null) {
        inboxStatus = if (status?.toLowerCase() == "click") {
          InboxStatus.CLICK
        } else if (status?.toLowerCase() == "delete" || status?.toLowerCase() == "deleted") {
          InboxStatus.DELETED
        } else {
          InboxStatus.SENT
        }
        context?.let {
          myInbox?.massiveEditNotifications(
            it,
            sendingIdList,
            inboxStatus,
            object : InboxBaseCallback(it) {
              override fun onSuccess() {
                ContextCompat.getMainExecutor(context).execute {
                  result.success(InboxParse.stringToMap(CALLBACK_MASSIVE_EDIT, "success"))
                }
              }

              override fun onError(
                errorId: Int?,
                errorMessage: String?,
                descriptionMessage: String?
              ) {
                ContextCompat.getMainExecutor(context).execute {
                  result.error(errorId.toString(), errorMessage, descriptionMessage)
                }
              }
            })
        }
      } else {
        context?.let{
          ContextCompat.getMainExecutor(context).execute {
            result.error("massiveEdit", "No status params", null)
            }
        }
      }
    }
  }

  fun getInfoFromNotification(
    myInbox: Inbox?,
    arguments: Any?,
    result: Result
  ){
    val MAP_SENDING_ID = "sendingId"
    var sendingId = 0
    arguments?.let{args ->
      val params = JSONObject(args as Map<*, *>)
      try {
        if (params.has(AUTH_VAR)) jsonAuth = params.getJSONObject(AUTH_VAR)
        if (params.has(MAP_SENDING_ID)) sendingId = params.getInt(MAP_SENDING_ID)
      } catch (ex: JSONException) {
        ex.printStackTrace()
      }
      context?.let {
        myInbox?.getInfoFromNotification(
          it,
          sendingId,
          object : InboxNotificationsCallback(it) {
            override fun onSuccess(inboxNotification: InboxNotification?) {
              if (inboxNotification != null) {
                ContextCompat.getMainExecutor(context).execute {
                  val inboxNotificationMap: MutableMap<String, Map<*,*>> = HashMap()
                  inboxNotificationMap[CALLBACK_GET_INFO_NOTIFICATION] = InboxParse.mapFromInboxNotification(inboxNotification)
                  result.success(inboxNotificationMap)
                }
              } else {
                ContextCompat.getMainExecutor(context).execute {
                  result.error("getInfoFromNotification", "inboxNotification is null", null)
                }
              }
            }

            override fun onFail(error: ErrorModel) {
              ContextCompat.getMainExecutor(context).execute {
                result.error(error.errorCode.name, error.errorMessage.errorMessage, null)
              }
            }
          })
      }
    }
  }

  fun modifyStatus(
    myInbox: Inbox?,
    arguments: Any?,
    result: Result
  ){
    val MAP_SENDING_ID = "sendingId"
    val MAP_STATUS = "status"
    var status: String? = null
    var sendingId = 0
    arguments?.let{args ->
      val params = JSONObject(args as Map<*, *>)
      try {
        if (params.has(MAP_SENDING_ID)) sendingId = params.getInt(MAP_SENDING_ID)
        if (params.has(MAP_STATUS)) status = params.getString(MAP_STATUS)
        if (params.has(AUTH_VAR)) jsonAuth = params.getJSONObject(AUTH_VAR)
      } catch (ex: JSONException) {
        ex.printStackTrace()
      }
      var inboxStatus: InboxStatus? = null
      if (status != null) {
        inboxStatus = if (status?.toLowerCase() == "click") {
          InboxStatus.CLICK
        } else if (status?.toLowerCase() == "delete" || status?.toLowerCase() == "deleted") {
          InboxStatus.DELETED
        } else {
          InboxStatus.SENT
        }
        context?.let {
          myInbox?.modifyStatusFromNotification(
            it,
            sendingId,
            inboxStatus,
            object : InboxNotificationsCallback(it) {
              override fun onSuccess(inboxNotification: InboxNotification?) {
                if (inboxNotification != null) {
                  ContextCompat.getMainExecutor(context).execute {
                    val inboxNotificationMap: MutableMap<String, Map<*,*>> = HashMap()
                    inboxNotificationMap[CALLBACK_MODIFY_STATUS] = InboxParse.mapFromInboxNotification(inboxNotification)
                    result.success(inboxNotificationMap)
                  }
                } else {
                  ContextCompat.getMainExecutor(context).execute {
                    result.error("getInfoFromNotification", "inboxNotification is null", null)
                  }
                }
              }

              override fun onFail(error: ErrorModel) {
                ContextCompat.getMainExecutor(context).execute {
                  result.error(error.errorCode.name, error.errorMessage.errorMessage, null)
                }
              }
            })
        }
      }
    }
  }

}
