//
//  IndigitallCustomerFlutterPlugin.m
//  indigitall_flutter_plugin
//
//  Created by indigitall on 15/10/21.
//

#import "IndigitallCustomerFlutterPlugin.h"
#import <Indigitall/INDevice.h>
#import "IndigitallCustomerFParse.h"
#import <IndigitallCore/INDefaults.h>

@implementation IndigitallCustomerFlutterPlugin

NSString * CALLBACK_CUSTOMER= @"callbackCustomer";
NSString * CALLBACK_CUSTOMER_FIELDS = @"callbackCustomerFields";

+ (void) getCustomer: (FlutterResult _Nonnull)result{
    [CustomerIndigitall getCustomerWithSuccess:^(INCustomer * _Nonnull customer) {
        result([IndigitallCustomerFParse jsonFromCustomer:customer]);
    } failed:^(INError * _Nonnull error) {
        [IndigitallCustomerFParse jsonFromError:error withResult:result];
    }];
  
}

+ (void) getCustomerInformation:(NSDictionary *_Nullable)params result: (FlutterResult _Nonnull)result{
    NSArray<NSString *>* fieldNamesList = params[@"fieldNamesList"];
    
    [CustomerIndigitall getCustomerInformationWithFieldNames:fieldNamesList success:^(NSArray<INCustomerField *> * _Nonnull customerFields) {
        result([IndigitallCustomerFParse jsonFromCustomerFields:customerFields]);
    } failed:^(INError * _Nonnull error) {
        [IndigitallCustomerFParse jsonFromError:error withResult:result];
    }];

}

+ (void) assignOrUpdateValueToCustomerFields:(NSDictionary *_Nullable)params result: (FlutterResult _Nonnull)result{
    
    [CustomerIndigitall updateValueToCustomerFieldsWithFields:params success:^(INCustomer * _Nonnull customer){
        result([IndigitallCustomerFParse jsonFromCustomer:customer]);
    } failed:^(INError * _Nonnull error) {
        [IndigitallCustomerFParse jsonFromError:error withResult:result];
    }];

}

+ (void) deleteValuesFromCustomerFields:(NSDictionary *_Nullable)params result: (FlutterResult _Nonnull)result{
    NSArray<NSString *>* fieldNamesList = params[@"fieldNamesList"];
    [CustomerIndigitall deleteValuesFromCustomerFieldsWithFieldNames:fieldNamesList success:^(INCustomer * _Nonnull customer) {
        result([IndigitallCustomerFParse jsonFromCustomer:customer]);
    } failed:^(INError * _Nonnull error) {
        [IndigitallCustomerFParse jsonFromError:error withResult:result];
    }];

}

+ (void) link:(NSDictionary *_Nullable)params result: (FlutterResult _Nonnull)result{
    NSString *code = params[@"externalCode"];
    if (code != nil){
        INDevice *device = [[[INDevice alloc]init]saveExternalCode:code];
        [INDefaults setExternalId:device.externalCode];
        [CustomerIndigitall linkCustomerWithChannel:[self getChannel:params[@"channel"]] success:^{
            result(@"");
        } failed:^(INError * _Nonnull error) {
            [IndigitallCustomerFParse jsonFromError:error withResult:result];
        }];
    }
    
}

+ (void) unlink:(NSDictionary *_Nullable)params result: (FlutterResult _Nonnull)result{
    [CustomerIndigitall unlinkCustomerWithChannel:[self getChannel:params[@"channel"]] success:^{
        result(@"");
    } failed:^(INError * _Nonnull error) {
        [IndigitallCustomerFParse jsonFromError:error withResult:result];
    }];    
}

+ (int) getChannel: (NSString *)channel {
    int channelType = 0;
    if ([channel isEqualToString:@"inapp"]) channelType = 1;
    if ([channel isEqualToString:@"chat"]) channelType = 2;
    return  channelType;
}


@end
