
#import "IndigitallInboxFlutterPlugin.h"
#import "IndigitallInboxFParse.h"
@implementation IndigitallInboxFlutterPlugin 

NSString * CALLBACK_GET_INBOX = @"callbackGetInbox";
NSString * CALLBACK_GET_MESSAGE_COUNT = @"callbackGetMessageCount";
NSString * CALLBACK_MASSIVE_EDIT = @"callbackMassiveEdit";
NSString * CALLBACK_GET_INFO_NOTIFICATION = @"callbackGetInfoNotification";
NSString * CALLBACK_MODIFY_STATUS = @"callbackModifyStatus";

@synthesize auth;

//+ (void) registerWithRegistrar:(NSObject<FlutterPluginRegistrar> *)registrar {
//    FlutterMethodChannel *channel = [FlutterMethodChannel methodChannelWithName:@"indigitall_inbox_flutter_plugin" binaryMessenger:registrar.messenger];
//    [channel setMethodCallHandler:^(FlutterMethodCall * _Nonnull call, FlutterResult  _Nonnull result) {
//
//    }];
//}

- (void) setAuth: (NSDictionary *)auth {
    self.auth = auth;
}

- (NSDictionary *)getAuthConfig {
    NSDictionary *dic = [NSDictionary new];
    if (auth != nil) dic = auth;
    return dic;
}

- (void) getInboxMessagesCountWithArguments:(NSDictionary *_Nullable)params result: (FlutterResult _Nonnull)result{
    if (params != nil) auth = params;
    [Inbox getMessagesCountWithSuccess:^(INInboxCounters * _Nonnull counters) {
        [IndigitallInboxFParse successWithInboxCounter:counters result: result];
    } onError:^(INError * _Nonnull error) {
        [IndigitallInboxFParse jsonFromError:error withResult:result];
    }];
}

- (void) getInboxWithArguments:(NSDictionary *_Nullable)params success: (void(^)(Inbox *inbox))onSuccess result:(FlutterResult _Nonnull)result {
    if (params != nil) auth = params;
    [Inbox getInboxWithSuccess:^(Inbox * _Nonnull inbox) {
        onSuccess(inbox);
        [IndigitallInboxFParse successWithInbox:inbox newNotifications:nil methodGetInbox:true result: result];
    } onError:^(INError * _Nonnull error) {
        [IndigitallInboxFParse jsonFromError:error withResult:result];
    }];
       
 
}

- (void) getNextPageWithArguments:(NSDictionary *_Nullable)params inbox: (Inbox *)inbox result:(FlutterResult _Nonnull)result {
    if (params != nil) auth = params;
    if (inbox != nil){
        [inbox getNextPageWithSuccess:^(Inbox * _Nonnull inbox, NSArray<INInboxNotification *> * _Nonnull newNotifications){
            [IndigitallInboxFParse successWithInbox:inbox newNotifications:newNotifications methodGetInbox:false result: result];
        } onError:^(INError * _Nonnull error) {
            [IndigitallInboxFParse jsonFromError:error withResult:result];
        }];
    }else{
        [IndigitallInboxFParse jsonFromError: [[INError alloc]initWithErrorCode:3020 descriptionMessage:@"getNextPage: inbox object is null"] withResult:result];
    }
}

- (void) massiveEditNotificationsWithSendingIdsList: (NSDictionary *)params inbox:(Inbox *)inbox result: (FlutterResult _Nonnull)result {
    NSArray<NSNumber *>* sendingIdList = params[@"sendingIdList"];
    NSString *status = params[@"status"];
    NSString *statusInbox;
    if (params[@"auth"] != nil) auth = params[@"auth"];
    if ([status isEqualToString:@"click"]){
        statusInbox = InboxStatus.CLICK;
    }else if ([status isEqualToString:@"deleted"]){
        statusInbox = InboxStatus.DELETED;
    }else{
        statusInbox = InboxStatus.SENT;
    }
    if (inbox != nil){
        [inbox massiveEditNotificationsWithSendingIdsList:sendingIdList status:statusInbox onSuccess:^{
            [IndigitallInboxFParse resultFromString:@"success" withAction:CALLBACK_MASSIVE_EDIT withResult:result];
        } onError:^(INError * _Nonnull error) {
            [IndigitallInboxFParse jsonFromError:error withResult:result];
        }];
    }else{
        [IndigitallInboxFParse jsonFromError: [[INError alloc]initWithErrorCode:3020 descriptionMessage:@"massiveEditNotificationsWithSendingIdsList: inbox object is null"] withResult:result];
    }
    
}

- (void) getInfoFromNotificationWithSendingId: (NSDictionary *)params inbox:(Inbox *)inbox result: (FlutterResult _Nonnull)result {
    if (inbox != nil){
        if (params[@"auth"] != nil) auth = params[@"auth"];
        NSNumber *sendingId = params[@"sendingId"];
        [inbox getInfoFromNotificationWithSendingId:sendingId onSuccess:^(INInboxNotification * _Nonnull inboxNotification) {
            [IndigitallInboxFParse successWithInboxNotification:inboxNotification callback:CALLBACK_GET_INFO_NOTIFICATION result: result];
        } onError:^(INError * _Nonnull error) {
            [IndigitallInboxFParse jsonFromError:error withResult:result];
        }];
    }else{
        [IndigitallInboxFParse jsonFromError: [[INError alloc]initWithErrorCode:3020 descriptionMessage:@"getInfoFromNotificationWithSendingId: inbox object is null"] withResult:result];
    }

}

- (void) modifyStatusFromNotificationWithSendingId: (NSDictionary *)params inbox:(Inbox *)inbox result: (FlutterResult _Nonnull)result {
    NSNumber * sendingId = params[@"sendingId"];
    NSString *status = params[@"status"];
    if (params[@"auth"] != nil) auth = params[@"auth"];
    if (inbox != nil){
        [inbox modifyStatusFromNotificationWithSendingId:sendingId status:status onSuccess:^(INInboxNotification * _Nonnull inboxNotification) {
            [IndigitallInboxFParse successWithInboxNotification:inboxNotification callback:CALLBACK_MODIFY_STATUS result: result];
        } onError:^(INError * _Nonnull error) {
            [IndigitallInboxFParse jsonFromError:error withResult:result];
        }];
    }else{
        [IndigitallInboxFParse jsonFromError: [[INError alloc]initWithErrorCode:3020 descriptionMessage:@"modifyStatusFromNotificationWithSendingId: inbox object is null"] withResult:result];
    }
    
}



@end
