//
//  IndigitallInboxParse.m
//  indigitall_flutter_plugin
//
//  Created by indigitall on 18/1/23.
//

#import "IndigitallInboxFParse.h"

@implementation IndigitallInboxFParse

NSString * CALLBACK_GET_INBOX_PARSE = @"callbackGetInbox";
NSString * CALLBACK_GET_MESSAGE_COUNT_PARSE = @"callbackGetMessageCount";
NSString * CALLBACK_GET_NEXT_PAGE_PARSE = @"callbackGetNextPage";

+ (NSDictionary *) jsonFromMessagesCounter: (INInboxCounters *) inboxCounter{
    NSMutableDictionary *obj = [[NSMutableDictionary alloc]init];
    if (inboxCounter != nil) {
        obj = [@{
            @"click" : @(inboxCounter.click) ?: 0,
            @"sent": @(inboxCounter.sent) ?: 0,
            @"deleted" : @(inboxCounter.deleted) ?: 0,
            @"lastAccess": inboxCounter.unread.lastAccess ?: @"",
            @"count": @(inboxCounter.unread.count) ?: 0
        } mutableCopy];
    }
    return [@{
        CALLBACK_GET_MESSAGE_COUNT_PARSE : [obj copy],
    } copy];
}

+ (NSDictionary *) jsonFromInbox: (Inbox *) inbox newNotifications:(NSArray<INInboxNotification *> *)newNotifications callback: (NSString *)callback {
    NSMutableDictionary *obj = [[NSMutableDictionary alloc]init];
    if (inbox != nil) {
        obj = [@{
            @"lastAccess" : inbox.lastAccess ?: @"",
            @"count": @(inbox.count) ?: 0,
            @"pageSize" : inbox.pageSize ?: 0,
            @"numPage": inbox.page ?: 0,
            @"notifications": [self jsonArrayFromInboxNotification: inbox.notifications] ?:@"",
            @"newNotifications": [self jsonArrayFromInboxNotification: newNotifications] ?:@""
        } mutableCopy];
    }
    
    return [@{
        callback : [obj copy],
    } copy];
}

+ (NSArray<NSString *>*) jsonArrayFromInboxNotification: (NSArray<INInboxNotification *>* _Nullable) inboxNotifications{
    NSMutableArray<NSString *>* array = [NSMutableArray new];
    @try{
        if (inboxNotifications && inboxNotifications.count > 0){
            for (int i=0;i<inboxNotifications.count;i++){
                NSMutableDictionary *obj = [[NSMutableDictionary alloc]init];
                if (inboxNotifications[i].idInboxNotification) [obj setValue:inboxNotifications[i].idInboxNotification forKey:@"id"];
                if (inboxNotifications[i].sentAt) [obj setValue:inboxNotifications[i].sentAt forKey:@"sentAt"];
                if (inboxNotifications[i].inboxStatus) [obj setValue:inboxNotifications[i].inboxStatus forKey:@"status"];
                if (inboxNotifications[i].sendingId) [obj setValue:inboxNotifications[i].sendingId forKey:@"sendingId"];
                if (inboxNotifications[i].campaignId) [obj setValue:inboxNotifications[i].campaignId forKey:@"campaignId"];
                if (inboxNotifications[i].message) [obj setValue:[self jsonFromPush:inboxNotifications[i].message] forKey:@"message"];
                if (inboxNotifications[i].category) [obj setValue:[self jsonFromCategory:inboxNotifications[i].category] forKey:@"category"];
             
                NSError * err;
                NSData * jsonData = [NSJSONSerialization dataWithJSONObject:obj options:0 error:&err];
                NSString * myString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
                if (myString != nil) [array addObject:myString];
                
            }
        }
    } @catch (NSException *exception) {
        NSLog(@"Error jsonArrayFromInboxNotification %@", exception);
    }
    if (array.count > 0){
        return [array copy];
    }else{
        return nil;
    }
}


+ (NSDictionary *) jsonFromCategory: (InboxCategory *) category{
    NSMutableDictionary *obj = [[NSMutableDictionary alloc]init];
    if (category != nil) {
        obj = [@{
            @"id" : @(category.idCategory) ?: 0,
            @"name": category.name ?: @"",
            @"code": category.code ?: @""
        } mutableCopy];
    }
    return [obj mutableCopy];
}

+ (NSDictionary *) jsonFromPush: (INPush *) message{
    NSMutableDictionary *obj = [[NSMutableDictionary alloc]init];
    if (message != nil) {
        if (message.title) [obj setValue:message.title forKey:@"title" ];
        if (message.body) [obj setValue:message.body forKey:@"body" ];
        if (message.image) [obj setValue:message.image forKey:@"image" ];
        if (message.gif) [obj setValue:message.gif forKey:@"gif" ];
        if (message.action) [obj setValue:[self jsonFromPushAction:message.action] forKey:@"action" ];
        if (message.buttons) [obj setValue: [self jsonArrayFromPushButtons:message.buttons] forKey:@"buttons" ];
        if (message.data) [obj setValue:message.data forKey:@"data" ];
        if (message.securedData) [obj setValue:message.securedData forKey:@"securedData" ];
    }
    return [obj copy];
}


+ (NSDictionary *) jsonFromPushAction: (INPushAction *) action{
    NSMutableDictionary *obj = [[NSMutableDictionary alloc]init];
    if (action != nil) {
        if (action.url) [obj setValue:action.url forKey:@"url"];
        if (action.app) [obj setValue:action.app forKey:@"app"];
        if (action.market) [obj setValue:action.market forKey:@"market"];
        if (action.share) [obj setValue:action.share forKey:@"share"];
        if (action.call) [obj setValue:action.call forKey:@"call"];
        if (action.noAction) [obj setValue:action.noAction forKey:@"noAction"];
        if (action.wallet) [obj setValue:action.wallet forKey:@"wallet"];
        if (action.destroy) [obj setValue:@(action.destroy) forKey:@"destroy"];
    }
    
    return [obj copy];
}

+ (NSArray<NSDictionary *>*) jsonArrayFromPushButtons: (NSArray<INPushButton *>*) buttons{
    NSMutableArray<NSDictionary *>* array = [NSMutableArray new];
    @try{
        if (buttons != nil){
            for (int i=0;i<buttons.count;i++){
                NSDictionary *obj = [[NSDictionary alloc]init];
                if (buttons[i].label) [obj setValue:buttons[i].label forKey:@"label"];
                if (buttons[i].action) [obj setValue:[self jsonFromPushAction: buttons[i].action] forKey:@"action"];
                if (obj != nil) [array addObject:obj];
            }
        }else{
            return nil;
        }
    } @catch (NSException *exception) {
        NSLog(@"Error jsonArrayFromPushButtons %@", exception);
    }
    return [array copy];
}


+ (void) successWithInbox: (Inbox *)inbox newNotifications:(NSArray<INInboxNotification *> *)newNotifications  methodGetInbox:(BOOL)methodGetInbox result: (FlutterResult _Nonnull)result {
    if (methodGetInbox){
        result([self jsonFromInbox: inbox newNotifications:newNotifications callback:CALLBACK_GET_INBOX_PARSE]);
    }else{
        result([self jsonFromInbox: inbox newNotifications:newNotifications callback:CALLBACK_GET_NEXT_PAGE_PARSE]);
    }
}

+ (void) successWithInboxCounter: (INInboxCounters *)counters  result: (FlutterResult _Nonnull)result {
    result([self jsonFromMessagesCounter: counters]);
    
}

+ (void) successWithInboxNotification: (INInboxNotification *)inboxNotification callback: (NSString *)callback result: (FlutterResult _Nonnull)result {
    if (inboxNotification != nil){
        NSArray<INInboxNotification *>* notifications = [NSArray new];
        notifications = [notifications arrayByAddingObject:inboxNotification];
        NSMutableDictionary *dic = [NSMutableDictionary new];
        @try {
            dic = [@{
            callback : [self jsonArrayFromInboxNotification:notifications].firstObject
            } mutableCopy];
        } @catch (NSException *exception) {
            NSLog(@"Error successWithInboxNotification %@", exception);
        }
        result([dic copy]);
    }else{
        [self jsonFromError: [[INError alloc]initWithErrorCode:3020 descriptionMessage:@"successWithInboxNotification: inboxNotification object is null"] withResult:result];
    }
}

+ (void) resultFromString: (NSString *)string withAction: (NSString *)action withResult: (FlutterResult _Nonnull) result {
    NSMutableDictionary *dic = [NSMutableDictionary new];
    @try {
        dic = [@{
        action : string
        } mutableCopy];
    } @catch (NSException *exception) {
        NSLog(@"Error resultFromString %@", exception);
    }
    result([dic copy]);
}


+ (void) jsonFromError: (INError *)error withResult: (FlutterResult _Nonnull) result {
    result([FlutterError errorWithCode:[NSString stringWithFormat:@"%u",error.errorCode] message:error.message details:nil]);
}

@end
