//
//  IndigitallPushParse.m
//  indigitall_flutter_plugin
//
//  Created by indigitall on 18/1/23.
//

#import "IndigitallPushFParse.h"

@implementation IndigitallPushFParse
+ (void)jsonFromDevice: (INDevice *)device withKey: (NSString *)key withResult: (FlutterResult _Nonnull)result{
    NSMutableDictionary *obj = [[NSMutableDictionary alloc]init];
    NSMutableDictionary *deviceDict = [[NSMutableDictionary alloc]init];
    if (device != nil) {
        obj = [@{
            @"enable" : device.enabled,
            @"deviceId": device.deviceID ?: @"",
            @"pushToken" : device.pushToken ?: @"",
            @"platform": device.platform ?: @"",
            @"version": device.sdkVersion  ?: @"",
            @"productName" : device.productName  ?: @"",
            @"productVersion" : device.productVersion  ?: @"",
            @"osName": device.osName  ?: @"",
            @"osVersion": device.osVersion  ?: @"",
            @"deviceBrand": device.deviceBrand ?: @"",
            @"deviceModel": device.deviceModel ?: @"",
            @"operator": device.carrier ?: @"",
            @"deviceType": device.deviceType ?: @"",
            @"appVersion": device.appVersion ?: @"",
            @"locale": device.locale ?: @"",
            @"timeZone": device.timeZone ?: @"",
            @"timeOffset": @(device.timeOffset) ?: 0,
            @"externalApps": [self jsonArrayFromExternalApps: device.externalApplications]
        } mutableCopy];
    }
    [deviceDict setValue:[obj copy] forKey:key];
    result([deviceDict copy]);
}

+ (NSArray<NSDictionary *>*) jsonArrayFromExternalApps: (NSArray<INCoreExternalApp *>*)externalApps{
    NSArray<NSDictionary *>* array = [NSArray new];
    @try{
        if (externalApps != nil){
            for (int i=0;i<externalApps.count;i++){
                NSDictionary *obj = [[NSDictionary alloc]init];
                obj = @{
                @"id" : @(externalApps[i].iNExternalAppId) ?: 0,
                @"name": externalApps[i].name ?: @"",
                @"code" : externalApps[i].iosCode ?: @"",
                };
                array = [array arrayByAddingObject:obj];
            }
        }
    } @catch (NSException *exception) {
        NSLog(@"Error jsonArrayFromExternalApps ");
    }
    return array;
}

+ (NSArray<NSDictionary *>*) jsonArrayFromTopics: (NSArray<INTopic *>*) topics{
    NSArray<NSDictionary *>* array = [NSArray new];
    @try{
        for (int i=0;i<topics.count;i++){
            NSDictionary *obj = [[NSDictionary alloc]init];
            obj = @{
                @"code" : topics[i].code ?: @"",
                @"name": topics[i].name ?: @"",
                @"parentCode" : topics[i].parentCode ?: @"",
                @"visible" :@(topics[i].visible) ?: false,
                @"subscribed" :@(topics[i].subscribed) ?: false
            };
            array = [array arrayByAddingObject:obj];
            
        }
    } @catch (NSException *exception) {
        NSLog(@"Error jsonArrayFromTopics %@", exception);
    }
    return array;
}


+ (void) resultFromString: (NSString *)string withAction: (NSString *)action withResult: (FlutterResult _Nonnull) result {
    NSMutableDictionary *dic = [NSMutableDictionary new];
    @try {
        dic = [@{
        action : string
        } mutableCopy];
    } @catch (NSException *exception) {
        result([FlutterError errorWithCode:action message:[NSString stringWithFormat:@"Error jsonFromError %@", exception] details:nil]);
    }
    result([dic copy]);
}

+ (void) resultFromTopic: (NSArray<NSDictionary *>*) topics key: (NSString *) key withResult: (FlutterResult _Nonnull) result {
    NSMutableDictionary *dict = [NSMutableDictionary new];
    [dict setValue:topics forKey:key];
    result([dict copy]);
}

+ (void) jsonFromError: (INError *)error withResult: (FlutterResult _Nonnull) result {
    result([FlutterError errorWithCode:[NSString stringWithFormat:@"%u",error.errorCode] message:error.message details:nil]);
}

+ (void) jsonFromStringError: (NSString *)message withResult: (FlutterResult _Nonnull) result {
    result([FlutterError errorWithCode:@"" message:message details:nil]);
}

@end
